searchData={"items":[{"type":"module","title":"opuntia","doc":"opuntia , traffic shapers for Erlang and Elixir This module implements the token-bucket traffic-shaping algorithm. The rate is given in t:tokens() per t:time_unit() and a bucket size. Resolution is in native unit times as described by erlang:monotonic_time/0 . The delay is always returned in milliseconds unit, as this is the unit receives and timers use in the BEAM.","ref":"opuntia.html"},{"type":"type","title":"opuntia.bucket_size/0","doc":"Maximum capacity of the bucket regardless of how much time passes.","ref":"opuntia.html#t:bucket_size/0"},{"type":"type","title":"opuntia.config/0","doc":null,"ref":"opuntia.html#t:config/0"},{"type":"type","title":"opuntia.delay/0","doc":"Number of milliseconds that is advise to wait after a shaping update.","ref":"opuntia.html#t:delay/0"},{"type":"function","title":"opuntia.new/1","doc":"Creates a new shaper according to the configuration. If zero is given, no shaper in created and any update action will always return zero delay; to configure a shaper it will need a config like #{ bucket_size =&gt; MaximumTokens , rate =&gt; Rate , time_unit =&gt; TimeUnit , start_full =&gt; Boolean } where TimeUnit is the time unit of measurement as defined by t:time_unit() Rate is the number of tokens per TimeUnit the bucket will grow with. MaximumTokens is the maximum number of tokens the bucket can grow. StartFull indicates if the shaper starts with the bucket full, or empty if not. So, for example, if we configure a shaper with the following: #{ bucket_size =&gt; 60000 , rate =&gt; 10 , time_unit =&gt; millisecond , start_full =&gt; true } it means that the bucket will allow 10 tokens per millisecond , up to 60000 tokens, regardless of how long it is left unused to charge: it will never charge further than 60000 tokens.","ref":"opuntia.html#new/1"},{"type":"function","title":"opuntia.peek/1","doc":"Peek currently available tokens.","ref":"opuntia.html#peek/1"},{"type":"type","title":"opuntia.rate/0","doc":"Number of tokens accepted per t:time_unit() .","ref":"opuntia.html#t:rate/0"},{"type":"type","title":"opuntia.shape/0","doc":"See new/1 for more details.","ref":"opuntia.html#t:shape/0"},{"type":"type","title":"opuntia.shaper/0","doc":"Shaper type","ref":"opuntia.html#t:shaper/0"},{"type":"type","title":"opuntia.time_unit/0","doc":"Supported shaping time units.","ref":"opuntia.html#t:time_unit/0"},{"type":"type","title":"opuntia.tokens/0","doc":"Unit element the shaper consumes, for example bytes or requests.","ref":"opuntia.html#t:tokens/0"},{"type":"function","title":"opuntia.update/2","doc":"Update shaper and return possible waiting time. This function takes the current shaper state, and the number of tokens that have been consumed, and returns a tuple containing the new shaper state, and a possibly non-zero number of unit times to wait if more tokens that the shaper allows were consumed.","ref":"opuntia.html#update/2"},{"type":"module","title":"opuntia_srv","doc":"Shared shapers.","ref":"opuntia_srv.html"},{"type":"type","title":"opuntia_srv.args/0","doc":null,"ref":"opuntia_srv.html#t:args/0"},{"type":"type","title":"opuntia_srv.gen_shape/0","doc":"This accepts a function that generates the shape, if such shape was too expensive to calculate. Note that for this server, only full buckets and in milliseconds are valid, due to the nature of gen_server call timeouts.","ref":"opuntia_srv.html#t:gen_shape/0"},{"type":"type","title":"opuntia_srv.key/0","doc":null,"ref":"opuntia_srv.html#t:key/0"},{"type":"type","title":"opuntia_srv.name/0","doc":null,"ref":"opuntia_srv.html#t:name/0"},{"type":"type","title":"opuntia_srv.opuntia_state/0","doc":null,"ref":"opuntia_srv.html#t:opuntia_state/0"},{"type":"function","title":"opuntia_srv.request_wait/4","doc":"Shapes the caller from executing the action, asynchronously This will do a gen_server:send_request/2 . Usual pattern applies to receive the matching continue.","ref":"opuntia_srv.html#request_wait/4"},{"type":"function","title":"opuntia_srv.reset_shapers/1","doc":"Ask server to forget all its shapers","ref":"opuntia_srv.html#reset_shapers/1"},{"type":"type","title":"opuntia_srv.seconds/0","doc":null,"ref":"opuntia_srv.html#t:seconds/0"},{"type":"type","title":"opuntia_srv.shape/0","doc":null,"ref":"opuntia_srv.html#t:shape/0"},{"type":"function","title":"opuntia_srv.start_link/2","doc":"Start-links a shaper server","ref":"opuntia_srv.html#start_link/2"},{"type":"function","title":"opuntia_srv.wait/4","doc":"Shapes the caller from executing the action This will do an actual blocking gen_server:call/3 .","ref":"opuntia_srv.html#wait/4"},{"type":"extras","title":"opuntia","doc":"# `opuntia`\n\n[![Actions Status](https://github.com/esl/opuntia/actions/workflows/ci.yml/badge.svg)](https://github.com/esl/opuntia/actions/workflows/ci.yml)\n[![codecov](https://codecov.io/gh/esl/opuntia/branch/main/graph/badge.svg)](https://codecov.io/gh/esl/opuntia)\n[![Hex](http://img.shields.io/hexpm/v/opuntia.svg)](https://hex.pm/packages/opuntia)\n\n`opuntia` is a basic set of tools for traffic shaping for erlang and elixir\n\nIt implements the [token bucket algorithm](https://en.wikipedia.org/wiki/Token_bucket).\n\nThere are two ways to use it, checking availability a priori or accepting a penalisation.\n\nAfter creating a bucket\n```erl\nBucket = opuntia:new(#{bucket_size => 10, rate => 1, time_unit => millisecond, start_full => true}),\n```\nyou can either consume all tokens queued and see the suggested delay, considering that this might\nallow you to consume at once much more than the bucket size:\n```erl\n{NewShaper, Delay} = opuntia:update(Shaper, 50),\ntimer:sleep(Delay), %% Will suggest to sleep 40ms\n```\nor you can first how many tokens are available for you to consume before doing so:\n```erl\nAllowed = opuntia:peek(Shaper),\nconsume_tokens(Allowed),\n{NewShaper, 0} = opuntia:update(Shaper), %% Will suggest no delay if you were diligent and consume less that adviced\n```","ref":"readme.html"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   Copyright 2022, Erlang Solutions LTD.\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"}],"proglang":"erlang","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}